      SUBROUTINE TYPE251(TIME,XIN,OUT,T,DTDT,PAR,INFO,ICNTRL,*)

C***************************************************************************
C.	THIS COMPONENT SIMULATES THE THERMAL PERFORMANCE OF A
C.	FLAT-PLATE SOLAR COLLECTOR USING THE MODEL DEVELOPED BY
C.	HOTTEL, WHILLIER, AND BLISS.
C	LAST MODIFIED 3/93 - JWT
C	3/04 - TPM - FOR TRNSYS 16
C	12/05 - DAA - Added units checking with RCHECK
C	02/07 - MRC - Simplified to Type h only
C***************************************************************************
C.
C.	PARAMETERS
C.	A	COLLECTOR AREA
C.	ALF	ABSORPTANCE OF THE COLLECTOR PLATE SURFACE
C.	AR	APERTURE AREA TO ABSORBER AREA
C.	CB	THERMAL CONDUCTANCE BETWEEN CELLS AND ABSORBER
C.	CPF	THERMAL CAPACITANCE OF THE COLLECTOR FLUID
C.	EP	THERMAL EMITTANCE OF THE COLLECTOR PLATE SURFACE 
C.	FE	APPROPRIATE FIN EFFICIENCY
C.	NG/XNG	NUMBER OF GLASS COVERS
C	TAU	COVER TRANSMITTANCE
C.	UB	BACK LOSS COEFFICIENT
C.	UF	FILM COEFFICIENT BETWEEN FLUID AND ABSORBER
C
C.	INPUT
C.	FLWRT	COLLECTOR FLUID FLOWRATE
C.	HR	TOTAL RADIATION INCIDENT ON THE TILTED COLLECTOR SURFACE
C.	TA	AMBIENT TEMPERATURE
C.	TILT	COLLECTOR SLOPE
C.	TIN	INLET FLUID TEMPERATURE
C.	V	VOLTAGE APPLIED TO ARRAY
C.	WIND	WINDSPEED
C.
C.	OUTPUT
C.	CURREN	ARRAY OUTPUT CURRENT
C.	QE	ELECTRICAL POWER OUTPUT
C.	QU	USEFUL ENERGY COLLECTION RATE PER UNIT AREA
C.	TAUALF	PRODUCT OF THE TRANSMITTANCE OF THE GLASS
C.		AND THE ABSORPTANCE OF THE COLLECTOR PLATE SURFACE
C.	TOUT	OUTLET FLUID TEMPERATURE
C.	TCELL	AVERAGE CELL TEMPERATURE
C.	TCELLR	CELL TEMPERATURE AT INLET
C.	UL	OVERALL ENERGY LOSS COEFFICIENT. SEE KLEIN
C.	ULO	APPARENT THERMAL LOSS COEFFICIENT
C.
C-----------------------------------------------------------------------------------------------------------------------
! Copyright  2004 Solar Energy Laboratory, University of Wisconsin-Madison. All rights reserved.
C-----------------------------------------------------------------------------------------------------------------------
C	USE STATEMENTS
	USE TrnsysFunctions
C-----------------------------------------------------------------------------------------------------------------------
C	REQUIRED BY THE MULTI-DLL VERSION OF TRNSYS
      !DEC$ATTRIBUTES DLLEXPORT :: TYPE251				!SET THE CORRECT TYPE NUMBER HERE

C	TRNSYS DECLARATIONS
	IMPLICIT NONE	
	DOUBLE PRECISION XIN,OUT,TIME,PAR,STORED,T,DTDT			
	INTEGER*4 INFO(15),NP,NI,NOUT,ND			
	INTEGER*4 NPAR,NIN,NDER,IUNIT,ITYPE			
	INTEGER*4 ICNTRL,NSTORED				
	CHARACTER*3 OCHECK,YCHECK				
C-----------------------------------------------------------------------------------------------------------------------
C	USER DECLARATIONS
	PARAMETER (NP=13,NI=7,NOUT=11,ND=0,NSTORED=0)
C-----------------------------------------------------------------------------------------------------------------------
C	REQUIRED TRNSYS DIMENSIONS
	DIMENSION XIN(NI),OUT(NOUT),PAR(NP),YCHECK(NI),OCHECK(NOUT),
     &	STORED(NSTORED),T(ND),DTDT(ND)
C-----------------------------------------------------------------------------------------------------------------------
C	DECLARATIONS AND DEFINITIONS FOR THE USER-VARIABLES
	DOUBLE PRECISION IC,K1,K2,EG,PI,SB,ATR,BTR,CTR,TAU040,REFIND,AR,
     & UBE,A,FP,CPF,ALF,UL,TAU,BR,TR,CELLPF,TAUALF,XNG,EP,XKL,
     & TAU60,FE,UB,CB,UF,UT,TIN,FLWRT,TA,TC,BA,HR,ETAR,ETAA,WIND,HBT,
     & HDT,THETA1,ICT,TILT,V,COSTH1,THETA2,COSTH2,HWIND,TM,TMC,TAC,F,C,
     & STF1,STF2,S,SINC,DENOM,TP,TCELLR,TCELL,TEMP,PR,BETA,PRS,ULAB,ULO,
     & FR,HRT,QU,QE,TOUT,TPR,PRSAVE,BETAS,TIME0,TFINAL,DELT
	INTEGER MODE,NPP,ITER,LUIN,NG
	DIMENSION ATR(3),BTR(3),CTR(3),TAU040(3)
	DATA EG/0.88/,PI/3.1415927/,SB/5.678E-08/
	DATA ATR/-2.9868,-1.4214,-0.74816/
	DATA BTR/-3.7360,-5.7356,-6.5262/
	DATA CTR/4.3541,5.7723,6.3769/
	DATA TAU040/0.92,0.845,0.785/
	DATA IUNIT/0/,REFIND/1.526/,NPP/0/,AR/1./,UBE/0./
C-----------------------------------------------------------------------------------------------------------------------
C	TRNSYS FUNCTIONS
	TIME0=getSimulationStartTime()
	TFINAL=getSimulationStopTime()
	DELT=getSimulationTimeStep()
C-----------------------------------------------------------------------------------------------------------------------
C	SET THE VERSION INFORMATION FOR TRNSYS
	IF(INFO(7).EQ.-2) THEN
		INFO(12)=16
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	DO ALL THE VERY LAST CALL OF THE SIMULATION MANIPULATIONS HERE
	IF (INFO(8).EQ.-1) THEN
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	PERFORM ANY "AFTER-ITERATION" MANIPULATIONS THAT ARE REQUIRED
	IF(INFO(13).GT.0) THEN
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C	DO ALL THE VERY FIRST CALL OF THE SIMULATION MANIPULATIONS HERE
	IF (INFO(7).EQ.-1) THEN
C       RETRIEVE THE UNIT NUMBER AND TYPE NUMBER FOR THIS COMPONENT FROM THE INFO ARRAY
		IUNIT=INFO(1)
		ITYPE=INFO(2)
C	SET SOME INFO ARRAY VARIABLES TO TELL THE TRNSYS ENGINE HOW THIS TYPE IS TO WORK
		INFO(6)=11				
		INFO(9)=1				
		INFO(10)=0				
C       SET THE REQUIRED # OF INPUTS, PARAMETERS AND DERIVATIVES
		MODE=PAR(1)
		NPAR=INFO(4)
		CALL TYPECK (1,INFO,7,NPAR,0)
		CALL RCHECK(INFO,YCHECK,OCHECK)
	RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C    DO THE INITIAL TIMESTEP MANIPULATIONS HERE
	IF (TIME.LT.(TIME0+DELT/2.D0)) THEN
		IUNIT=INFO(1)
		ITYPE=INFO(2)
		ITER=0
		MODE=PAR(1)
		A=PAR(2)
		AR=PAR(3)
		CPF=PAR(4)
		ALF=PAR(5)
		NPP=1
		FE=PAR(6)
		UB=PAR(7)
		CB=PAR(8)
		UF=PAR(9)
		UF=UF*(1.+FE)
		IF (CB.LT.1.E-5) CALL TYPECK (4,INFO,0,0,0)
		TAU=PAR(10)
		TAUALF=TAU*ALF
		XNG=PAR(11)
		EP=PAR(12)
		LUIN=-1
		IF(INFO(4).EQ.13) LUIN=PAR(13)
		CALL SOLARCELL (TCELLR,TEMP,SINC,PR,BETA,IC,V,ITER,NPP,-1,A,
     & AR,MODE,LUIN)
		RETURN 1
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C    RE-READ THE PARAMETERS IF ANOTHER UNIT OF THIS TYPE HAS BEEN CALLED
	IF(INFO(1).NE.IUNIT) THEN
		IUNIT=INFO(1)
		ITYPE=INFO(2)
		ITER=0
		MODE=PAR(1)
		A=PAR(2)
		AR=PAR(3)
		CPF=PAR(4)
		ALF=PAR(5)
		NPP=1
		FE=PAR(6)
		UB=PAR(7)
		CB=PAR(8)
		UF=PAR(9)
		UF=UF*(1.+FE)
		IF (CB.LT.1.E-5) CALL TYPECK (4,INFO,0,0,0)
		TAU=PAR(10)
		TAUALF=TAU*ALF
		XNG=PAR(11)
		EP=PAR(12)
		LUIN=-1
		IF(INFO(4).EQ.13) LUIN=PAR(13)
	ENDIF
C-----------------------------------------------------------------------------------------------------------------------
C    RETRIEVE THE CURRENT VALUES OF THE INPUTS
	TIN=XIN(1)
	TM=TIN
	FLWRT=XIN(2)
	TILT=XIN(3)
	V=XIN(4)
	TA=XIN(5)
	HR=XIN(6)
	WIND=XIN(7)
	HWIND=5.7+3.8*WIND
	ICT=0
25	CONTINUE
	IF (ITER.EQ.0) ICT=ICT+1
	IF (ICT.GT.2) GO TO 45
	TMC=TM+273.15
	TAC=TA+273.15
	IF (TMC.LE.TAC) TMC=TAC+1.0
	F=(1.0-0.04*HWIND+5.0E-04*HWIND*HWIND)*(1.0+0.091*XNG)
	IF (XNG.LT..5) F=1.
	STF1=365.9/TMC*((TMC-TAC)/(XNG+F))**0.33
	STF1=XNG/STF1+1.0/HWIND
	STF1=1.0/STF1
	STF2=1.0/(EP+0.05*XNG*(1.0-EP))+(2.*XNG+F-1.)/EG-XNG
	STF2=SB*(TMC*TMC+TAC*TAC)*(TMC+TAC)/STF2
	UL=(STF1+STF2)*3.6+UBE
	S=HR*TAUALF*AR
	SINC=S/ALF
	UT=UL
	DENOM=UT*(CB+UF)+CB*UF
	K1=CB/DENOM
	IF (ICT.NE.1) GO TO 27
	TP=K1*(S-UT*(TIN-TA))+TIN
	TCELLR=UF*(TP-TIN)/CB+TP
	IF (FLWRT.LT.1.E-5) THEN
		TCELLR=(S+TA*(UT+CB*UB/(CB+UB)))/(UT+CB-CB**2/(CB+UB))
	ENDIF
	TCELL=TCELLR
27	TEMP=TCELL
	CALL SOLARCELL (TCELLR,TEMP,SINC,PR,BETA,IC,V,ITER,NPP,INFO(7),A,
     & AR,MODE,LUIN,*28)
	CALL LINKCK('TYPE50','SOLARCEL',1,99)
28	K2=1.+K1*PR*BETA*(CB+UF)/CB
	IF (S.LT.1.E-5) PRS=0.
	IF (S.GE.1.E-5) PRS=PR/S
	FP=UF*K1*(1.-PRS*(1.+BETA*(TA-TCELLR)))/K2
	ULAB=UF*K1*(UT+PR*BETA)/(K2*FP)
	UL=ULAB/AR
	ULO=UT/AR
	IF (FLWRT-1.E-5) 34,34,30
30	IF ((FP*UL*A/(FLWRT*CPF)).LT.0.01) GO TO 31
	FR=FLWRT*CPF*(1.0-DEXP(-FP*UL*A/(FLWRT*CPF)))/(A*UL)
	GO TO 32
31	FR=FP
32	HRT=HR
	QU=FR*(HRT*TAUALF-UL*(TIN-TA))
	QE=PR*(1.+BETA*(TCELL-TCELLR))/AR
	TOUT=QU/FLWRT*A/CPF+TIN
	GO TO 36
34	QU=0.0
	QE=PR/AR
	TCELLR=(S-PR+TA*(UT+CB*UB/(CB+UB)))/(UT+CB-CB**2/(CB+UB))
	TCELL=TCELLR
	TEMP=TCELL
	TOUT=TCELL*(1.-UB/(UB+CB))+TA*UB/(UB+CB)
	FLWRT=0.
	GO TO 39
36	TM=(TIN+TOUT)/2.0
	TP=QU*AR/UF+TIN
	TCELLR=UF*(TP-TIN)/CB+TP
	TPR=QU*AR/UF+TM
	TCELL=UF*(TPR-TM)/CB+TPR
	TM=TCELL
	TEMP=TCELL
39	PRSAVE=PR
	BETAS=BETA
	ITER=1
	CALL SOLARCELL (TCELLR,TEMP,SINC,PR,BETA,IC,V,ITER,NPP,INFO(7),A,
     & AR,MODE,LUIN,*391)
	CALL LINKCK('TYPE50','SOLARCEL',1,99)
391	IF (PR.LT.1.E-5.AND.PRSAVE.LT.1.E-5) GO TO 40
	IF (PR.LT.1.E-5) GO TO 25
	IF (DABS((PR-PRSAVE)/PR).GT.0.05) GO TO 25
40	IF (BETA.LT.1.E-5.AND.BETAS.LT.1.E-5) GO TO 41
	IF (BETA.LT.1.E-5) GO TO 25
	IF (DABS((BETA-BETAS)/BETA).GT.0.05) GO TO 25
41	ITER=0
	GO TO 25
45	OUT(1)=QU*A	
	OUT(2)=QE*A
	OUT(3)=TOUT
	OUT(4)=FLWRT
	OUT(5)=ULO
	OUT(6)=UL
	OUT(7)=TAUALF
	OUT(8)=TCELL
	OUT(9)=TCELLR
	OUT(10)=V
	OUT(11)=QE*A/(V*3.6)
	RETURN 1
	END
C-----------------------------------------------------------------------------------------------------------------------
